package DE_EPAGES::ShoppingList::t::ShoppingList;
use base DE_EPAGES::Database::API::Test;

use Test::More tests => 12;
use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw ( DeleteObject LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

my $ROOTALIAS = 'Test';

sub test {

    my $Shop = LoadRootObject->child($ROOTALIAS)->child('TestShop');
    ok( $Shop->id, 'create shop ' . $Shop->id );

    my $User = $Shop->child('Users')->child('Tester');
    my $Folder = $User->folder('ShoppingLists');
    my $ShoppingList = $Folder->insertChild('ShoppingList', {
        Alias => 'MyShoppingList',
        User => $User
    });
    ok( $ShoppingList->instanceOf('ShoppingList'), 'Create ShoppingList' );
    ok( $ShoppingList->get('User')->id == $User->id, 'ShoppingList.User' );
    ok( !$ShoppingList->hasChildren, 'New ShoppingList is empty' );

    my $Product = $Shop->path('Products/MyPetrol');
    $ShoppingList->addItem($Product);
    ok( $ShoppingList->hasChildren, 'New ShoppingList is not empty' );
    my $aItems = $ShoppingList->children;
    is( scalar @$aItems, 1, 'shopping list has 1 item' );
    my $Item = $aItems->[0];
    ok( $Item->instanceOf('ShoppingListItem'), 'is an instance of class ShoppingListItem' );
    is( $Item->get('Product')->id, $Product ->id, 'item refers to the product' );

    my $MasterProduct = $Shop->path('Products/MyJacket');
    eval { $ShoppingList->addItem($MasterProduct) };
    ok( ExistsError(), 'Can \'t add master product to shopping list' );
    is( GetError()->code, 'NoMasterProductInShoppingList', 'error code' );

    my $SubProduct = $Shop->path('Products/MyJacket/SubProducts/JacketRed4');
    $ShoppingList->addItem($SubProduct);
    $aItems = $ShoppingList->children;
    is( scalar @$aItems, 2, 'shopping list now has 2 items' );
    $Item = $aItems->[0];
    # shhopping list items are sorted by the Alias of the products
    is( $Item->get('Product')->id, $SubProduct->id, 'added the sub product before MyPetrol' );
}

sub init {
    my $System = LoadRootObject();
    my $TEST = $System->existsChild($ROOTALIAS)
        ? $System->child($ROOTALIAS)
        : $System->insertChild('Object', {'Alias'=>$ROOTALIAS});
    ImportObjects('import.xml', $TEST->pathString);
    return 1;
}

sub cleanup {
    my $System = LoadRootObject();
    return unless $System->existsChild($ROOTALIAS);

    if( $ENV{'SKIP_CLEANUP'} ) {
        my $TestRoot = $System->child($ROOTALIAS);
        if( $TestRoot->existsChild('TestShop') ) {
            my $Shop = $TestRoot->child('TestShop');
            my $ShoppingListID = $Shop->findPath('Users/Tester/ShoppingLists/MyShoppingList');
            DeleteObject($ShoppingListID) if defined $ShoppingListID;
        }
    }
    else {
        $System->child($ROOTALIAS)->delete;
    }
}

__PACKAGE__->runScript;
